;+
; NAME:
;  photcal
; PURPOSE:   (one line only)
;  Photometric calibration using two lists of partially overlapping sources.
; DESCRIPTION:
; CATEGORY:
;  Photometry
; CALLING SEQUENCE:
;  photcal,ra,dec,imag,imagerr,catra,catdec,catmag,photzp
; INPUTS:
;  ra   - J2000 right ascension of source (usually from a CCD image)
;  dec  - J2000 declination of source (usually from a CCD image)
;  imag - instrumental magnitude
;  imagerr - uncertainty on the instrument magnitude
;  catra   - J2000 right ascension of the catalog (known) source.
;  catdec  - J2000 declination of the catalog (known) source.
;  catmag  - Standard catalog magnitude
;  catmagerr  - Uncertainty of the standard catalog magnitude
; OPTIONAL INPUT PARAMETERS:
; KEYWORD INPUT PARAMETERS:
; OUTPUTS:
;  photzp  - Photometric zero-point that when added to the instrumental
;              magnitude will return the standard magnitude in the system
;              of the catalog.
; KEYWORD OUTPUT PARAMETERS:
; COMMON BLOCKS:
; SIDE EFFECTS:
; RESTRICTIONS:
; PROCEDURE:
; MODIFICATION HISTORY:
;  Written by Marc W. Buie, Lowell Observatory, 2007/02/01
;-
pro photcal,ra,dec,imag,imagerr,catra,catdec,catmag,catmagerr, $
            ngood,photzp,photzperr,magrange

   self='photcal: '
   if badpar(ra,[4,5],[0,1],caller=self+'(ra) ') then return
   if badpar(dec,[4,5],[0,1],caller=self+'(dec) ') then return
   if badpar(imag,[4,5],[0,1],caller=self+'(imag) ') then return
   if badpar(imagerr,[4,5],[0,1],caller=self+'(imagerr) ') then return
   if badpar(catra,[4,5],[0,1],caller=self+'(catra) ') then return
   if badpar(catdec,[4,5],[0,1],caller=self+'(catdec) ') then return
   if badpar(catmag,[4,5],[0,1],caller=self+'(catmag) ') then return
   if badpar(catmagerr,[4,5],[0,1],caller=self+'(catmag) ') then return

   dcr = 1.5 / 3600.0 * !pi / 180.0
   srcor,catra,catdec,ra,dec,dcr,ind1,ind2,option=1

   if n_elements(ind1) ne n_elements(ind2) then begin
print,'This cannot happen.'
stop
   endif

   if ind1[0] lt 0 then begin
      ngood=0
      photzp=0.
      photzperr=0.
      magrange=[0.,0.]
      return
   endif

   zpall = catmag[ind1]-imag[ind2]
   zpallerr = sqrt(catmagerr[ind1]^2+imagerr[ind2]^2)
   bad=bytarr(n_elements(zpall))
   robomean,zpall,2.0,0.5,photzp,stdmean=photzperr,bad=bad
   z=where(bad eq 0,ngood)
   magrange=maxmin(catmag[ind1[z]])

   if n_elements(ind1) gt 1 then begin
      setwin,0
      ploterror,catmag[ind1],imag[ind2],catmagerr[ind1],imagerr[ind2], $
         psym=8,xr=maxmin(catmag[ind1]),yr=maxmin(imag[ind2]),/nohat

      setwin,1
      ploterror,catmag[ind1],zpall,catmagerr[ind1],zpallerr,psym=8, $
         xr=maxmin(catmag[ind1]),yr=maxmin(zpall),/nohat

      oplot,catmag[ind1[z]],zpall[z],psym=8,color='ff00ff'xl
   endif

end
